@@ -2,8 +2,6 @@
 
 set -o errexit -o pipefail -o noclobber -o nounset
 
-GLIB_TAG="2.78.0"
-GOBJECT_INTROSPECTION_TAG="1.78.0"
 FREETYPE2_TAG="VER-2-13-2"
 FONTCONFIG_TAG="2.14.2"
 CAIRO_TAG="1.18.0"
@@ -56,19 +54,6 @@ fi
 # The following assumes that $PREFIX has been set
 source ci/setup-dependencies-env.sh
 
-git clone --depth 1 --branch $GLIB_TAG https://gitlab.gnome.org/GNOME/glib
-cd glib
-meson setup _build --prefix $PREFIX $MESON_FLAGS
-meson compile -C _build
-meson install -C _build
-
-cd ..
-git clone --depth 1 --branch $GOBJECT_INTROSPECTION_TAG https://gitlab.gnome.org/GNOME/gobject-introspection
-cd gobject-introspection
-meson setup _build --prefix $PREFIX $MESON_FLAGS
-meson compile -C _build
-meson install -C _build
-
 cd ..
 git clone --depth 1 --branch $FREETYPE2_TAG https://gitlab.freedesktop.org/freetype/freetype
 cd freetype
