@@ -13,16 +13,14 @@ use crate::element::{set_attribute, ElementTrait};
 use crate::error::*;
 use crate::iri::Iri;
 use crate::is_element_of_type;
-use crate::layout::{self, Layer, LayerKind, Marker, Shape, StackingContext, Stroke};
+use crate::layout::{Layer, LayerKind, Marker, Shape, StackingContext, Stroke};
 use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow};
-use crate::paint_server::PaintSource;
 use crate::parsers::{optional_comma, Parse, ParseValue};
 use crate::path_builder::{LargeArc, Path as SvgPath, PathBuilder, Sweep};
 use crate::properties::ComputedValues;
 use crate::rsvg_log;
 use crate::session::Session;
-use crate::transform::Transform;
 use crate::xml::Attributes;
 
 #[derive(PartialEq)]
@@ -46,41 +44,14 @@ trait BasicShape {
     fn make_shape(&self, params: &NormalizeParams, values: &ComputedValues) -> ShapeDef;
 }
 
-fn validate_path(
-    path: &Rc<SvgPath>,
-    viewport: &Viewport,
-    normalize_values: &NormalizeValues,
-    stroke_paint: &PaintSource,
-    fill_paint: &PaintSource,
-    transform: &Transform,
-) -> Result<layout::Path, InternalRenderingError> {
-    if path.has_unsuitable_coordinates(transform) {
-        return Ok(layout::Path::Invalid(String::from(
-            "path has coordinates that are unsuitable for Cairo",
-        )));
-    }
-
-    let extents = compute_path_extents(path)?;
-    let stroke_paint = stroke_paint.to_user_space(&extents, viewport, normalize_values);
-    let fill_paint = fill_paint.to_user_space(&extents, viewport, normalize_values);
-
-    Ok(layout::Path::Validated {
-        path: Rc::clone(path),
-        extents,
-        stroke_paint,
-        fill_paint,
-    })
-}
-
 fn draw_basic_shape(
     basic_shape: &dyn BasicShape,
     node: &Node,
     acquired_nodes: &mut AcquiredNodes<'_>,
     cascaded: &CascadedValues<'_>,
     viewport: &Viewport,
-    draw_ctx: &mut DrawingCtx,
-    clipping: bool,
-) -> Result<BoundingBox, InternalRenderingError> {
+    session: &Session,
+) -> Result<Layer, InternalRenderingError> {
     let values = cascaded.get();
     let params = NormalizeParams::new(values, viewport);
     let shape_def = basic_shape.make_shape(&params, values);
@@ -90,8 +61,6 @@ fn draw_basic_shape(
 
     let stroke = Stroke::new(values, &params);
 
-    let session = draw_ctx.session();
-
     let stroke_paint = values.stroke().0.resolve(
         acquired_nodes,
         values.stroke_opacity().0,
@@ -146,26 +115,21 @@ fn draw_basic_shape(
         context_fill: fill_paint.clone(),
     };
 
+    let extents = compute_path_extents(&shape_def.path)?;
+
     let normalize_values = NormalizeValues::new(values);
 
-    let path = validate_path(
-        &shape_def.path,
-        viewport,
-        &normalize_values,
-        &stroke_paint,
-        &fill_paint,
-        &draw_ctx.get_transform(),
-    )?;
-
-    if let layout::Path::Invalid(ref reason) = path {
-        rsvg_log!(session, "will not render {node}: {reason}");
-    }
+    let stroke_paint = stroke_paint.to_user_space(&extents, viewport, &normalize_values);
+    let fill_paint = fill_paint.to_user_space(&extents, viewport, &normalize_values);
 
     let shape = Box::new(Shape {
-        path,
+        path: shape_def.path,
+        extents,
         is_visible,
         paint_order,
         stroke,
+        stroke_paint,
+        fill_paint,
         fill_rule,
         clip_rule,
         shape_rendering,
@@ -176,7 +140,7 @@ fn draw_basic_shape(
 
     let elt = node.borrow_element();
     let stacking_ctx = StackingContext::new(
-        draw_ctx.session(),
+        session,
         acquired_nodes,
         &elt,
         values.transform(),
@@ -184,16 +148,34 @@ fn draw_basic_shape(
         values,
     );
 
-    let layer = Layer {
+    Ok(Layer {
         kind: LayerKind::Shape(shape),
         stacking_ctx,
-    };
-
-    draw_ctx.draw_layer(&layer, acquired_nodes, clipping, viewport)
+    })
 }
 
 macro_rules! impl_draw {
     () => {
+        fn layout(
+            &self,
+            node: &Node,
+            acquired_nodes: &mut AcquiredNodes<'_>,
+            cascaded: &CascadedValues<'_>,
+            viewport: &Viewport,
+            draw_ctx: &mut DrawingCtx,
+            _clipping: bool,
+        ) -> Result<Option<Layer>, InternalRenderingError> {
+            draw_basic_shape(
+                self,
+                node,
+                acquired_nodes,
+                cascaded,
+                viewport,
+                &draw_ctx.session().clone(),
+            )
+            .map(Some)
+        }
+
         fn draw(
             &self,
             node: &Node,
@@ -203,15 +185,10 @@ macro_rules! impl_draw {
             draw_ctx: &mut DrawingCtx,
             clipping: bool,
         ) -> Result<BoundingBox, InternalRenderingError> {
-            draw_basic_shape(
-                self,
-                node,
-                acquired_nodes,
-                cascaded,
-                viewport,
-                draw_ctx,
-                clipping,
-            )
+            self.layout(node, acquired_nodes, cascaded, viewport, draw_ctx, clipping)
+                .and_then(|layer| {
+                    draw_ctx.draw_layer(layer.as_ref().unwrap(), acquired_nodes, clipping, viewport)
+                })
         }
     };
 }
